@@ -1,6 +1,9 @@
 use strict;
 use warnings;
 package Plack::App::DAIA::Validator;
+{
+  $Plack::App::DAIA::Validator::VERSION = '0.2';
+}
 #ABSTRACT: DAIA validator and converter
 
 use CGI qw(:standard);
@@ -9,6 +12,22 @@ use Encode;
 use parent 'Plack::App::DAIA';
 use Plack::Util::Accessor qw(xsd xslt warnings);
 
+our ($FORMATS, $GRAPHVIZ);
+BEGIN {
+    $FORMATS = { 'html'=>'HTML','json'=>'DAIA/JSON','xml'=>'DAIA/XML' };
+    # optionally add DAIA/RDF
+    eval "use RDF::Trine::Serializer";
+    my @names = eval "RDF::Trine::Serializer->serializer_names" unless $@;
+    unless ($@) {
+        $FORMATS->{$_} = "DAIA/RDF ($_)" for @names;
+    }
+    eval "use RDF::Trine::Exporter::GraphViz";
+    $GRAPHVIZ = 'RDF::Trine::Exporter::Graphviz' unless $@;
+    unless ($@) {
+        $FORMATS->{$_} = "DAIA/RDF graph ($_)" for qw(dot svg);
+    }
+}
+
 sub call {
     my ($self, $env) = @_;
     my $req = Plack::Request->new($env);
@@ -25,7 +44,7 @@ sub call {
     my $xsd = $self->xsd;
 
     my $informat  = lc($req->param('in'));
-    my $outformat = lc($req->param('out')) || lc($req->param('format'));
+    my $outformat = lc($req->param('out')) || lc($req->param('format')) || 'html';
 
     my $callback  = $req->param('callback') || ""; 
     $callback = "" unless $callback =~ /^[a-z][a-z0-9._\[\]]*$/i;
@@ -50,11 +69,11 @@ sub call {
         $daia = shift @daiaobjs;
     }
 
-    if ( $outformat =~ /^(json|xml)$/ ) {
+    if ( $FORMATS->{$outformat} and $outformat ne 'html' ) {
         $daia = DAIA::Response->new() unless $daia;
-        $daia->addMessage(error(500,'en' => $error)) if $error;
-        return $self->serialize( 200, $daia, $outformat, $req->param('callback') );
-    } elsif ( $outformat and $outformat ne 'html' ) {
+        $daia->addError( 500, 'en' => $error ) if $error;
+        return $self->as_psgi( 200, $daia, $outformat, $req->param('callback') );
+    } elsif ( $outformat ne 'html' ) {
         $error = "Unknown output format - using HTML instead";
     }
 
@@ -95,8 +114,6 @@ sub call {
 <form method="post" accept-charset="utf-8" action="">
 HTML
 
-    # TODO: current value of informat/outformat
-
     $html .= $msg . $error .
      fieldset(label('Input: ',
             popup_menu('in',['','json','xml'],'',
@@ -108,13 +125,17 @@ HTML
       ).
       fieldset(
         label('Output: ',
-            popup_menu('out',['html','json','xml'],'html',
-                       {'html'=>'HTML','json'=>'DAIA/JSON','xml'=>'DAIA/XML'})
+            popup_menu('out',
+                [ sort { $FORMATS->{$a} cmp $FORMATS->{$b} } keys %$FORMATS ], 
+                $outformat, $FORMATS )
         ), '&#xA0;', 
         label('JSONP Callback: ', textfield(-name=>'callback',-value=>$callback))
       ).
       fieldset('<input type="submit" value="Convert" class="submit" />')
     ;
+    if ($GRAPHVIZ && $url && !$data) {
+       $html .= "<fieldset>See RDF graph <a href=\"?url=$eurl&format=svg\">as SVG</a></fieldset>";
+    }
     $html .= '</form>';
 
     if ($daia) {
@@ -156,10 +177,9 @@ HTML
       $html .= "</div>";
     }
 
-    my $VERSION = $DAIA::VERSION;
     $html .= <<HTML;
 <div class='footer'>
-Based on <a href='http://search.cpan.org/perldoc?Plack::App::DAIA'>Plack::App::DAIA</a> $VERSION.
+Based on <a href='http://search.cpan.org/perldoc?Plack::App::DAIA'>Plack::App::DAIA</a> ${DAIA::VERSION}.
 Visit the <a href="http://github.com/gbv/daia/">DAIA project at github</a> for sources and details. 
 </div></body>
 HTML
@@ -169,6 +189,18 @@ HTML
 
 1;
 
+
+__END__
+=pod
+
+=head1 NAME
+
+Plack::App::DAIA::Validator - DAIA validator and converter
+
+=head1 VERSION
+
+version 0.2
+
 =head1 SYNOPSIS
 
     use Plack::Builder;
@@ -182,14 +214,29 @@ HTML
             warnings => 1
         );
     };
-    
+
 =head1 DESCRIPTION
 
 This module provides a simple L<DAIA> validator and converter as PSGI web
 application.
 
+To support fetching from DAIA Servers via HTTPS you might need to install
+L<LWP::Protocol::https> version 6.02 or higher.
+
 =head1 CONFIGURATION
 
 All configuration parameters (C<xsd>, C<xslt>, and C<warnings>) are optional.
 
+=head1 AUTHOR
+
+Jakob Voss
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2011 by Jakob Voss.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
+
